==> create-fly-1.21.8-6.0.8-2-sources.jar: com/zurrtum/create/client/content/kinetics/steamEngine/SteamEngineValueBox.java <==
package com.zurrtum.create.client.content.kinetics.steamEngine;

import com.zurrtum.create.catnip.math.AngleHelper;
import com.zurrtum.create.catnip.math.Pointing;
import com.zurrtum.create.catnip.math.VecHelper;
import com.zurrtum.create.client.flywheel.lib.transform.TransformStack;
import com.zurrtum.create.client.foundation.blockEntity.behaviour.ValueBoxTransform;
import com.zurrtum.create.content.kinetics.steamEngine.SteamEngineBlock;
import net.minecraft.class_1936;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2350.class_2351;
import net.minecraft.class_243;
import net.minecraft.class_2680;
import net.minecraft.class_4587;

public class SteamEngineValueBox extends ValueBoxTransform.Sided {

    @Override
    protected boolean isSideActive(class_2680 state, class_2350 side) {
        class_2350 engineFacing = SteamEngineBlock.getFacing(state);
        if (engineFacing.method_10166() == side.method_10166())
            return false;

        float roll = 0;
        for (Pointing p : Pointing.values())
            if (p.getCombinedDirection(engineFacing) == side)
                roll = p.getXRotation();
        if (engineFacing == class_2350.field_11036)
            roll += 180;

        boolean recessed = roll % 180 == 0;
        if (engineFacing.method_10166() == class_2351.field_11052)
            recessed ^= state.method_11654(SteamEngineBlock.field_11177).method_10166() == class_2351.field_11048;

        return !recessed;
    }

    @Override
    public class_243 getLocalOffset(class_1936 level, class_2338 pos, class_2680 state) {
        class_2350 side = getSide();
        class_2350 engineFacing = SteamEngineBlock.getFacing(state);

        float roll = 0;
        for (Pointing p : Pointing.values())
            if (p.getCombinedDirection(engineFacing) == side)
                roll = p.getXRotation();
        if (engineFacing == class_2350.field_11036)
            roll += 180;

        float horizontalAngle = AngleHelper.horizontalAngle(engineFacing);
        float verticalAngle = AngleHelper.verticalAngle(engineFacing);
        class_243 local = VecHelper.voxelSpace(8, 14.5, 9);

        local = VecHelper.rotateCentered(local, roll, class_2351.field_11051);
        local = VecHelper.rotateCentered(local, horizontalAngle, class_2351.field_11052);
        local = VecHelper.rotateCentered(local, verticalAngle, class_2351.field_11048);

        return local;
    }

    @Override
    public void rotate(class_1936 level, class_2338 pos, class_2680 state, class_4587 ms) {
        class_2350 facing = SteamEngineBlock.getFacing(state);

        if (facing.method_10166() == class_2351.field_11052) {
            super.rotate(level, pos, state, ms);
            return;
        }

        float roll = 0;
        for (Pointing p : Pointing.values())
            if (p.getCombinedDirection(facing) == getSide())
                roll = p.getXRotation();

        float yRot = AngleHelper.horizontalAngle(facing) + (facing == class_2350.field_11033 ? 180 : 0);
        TransformStack.of(ms).rotateYDegrees(yRot).rotateXDegrees(facing == class_2350.field_11033 ? -90 : 90).rotateYDegrees(roll);
    }

    @Override
    protected class_243 getSouthLocation() {
        return class_243.field_1353;
    }

}

==> create-fly-1.21.10-6.0.8-4-sources.jar: com/zurrtum/create/client/content/kinetics/steamEngine/SteamEngineValueBox.java <==
package com.zurrtum.create.client.content.kinetics.steamEngine;

import com.zurrtum.create.catnip.math.AngleHelper;
import com.zurrtum.create.catnip.math.Pointing;
import com.zurrtum.create.catnip.math.VecHelper;
import com.zurrtum.create.client.flywheel.lib.transform.TransformStack;
import com.zurrtum.create.client.foundation.blockEntity.behaviour.ValueBoxTransform;
import com.zurrtum.create.content.kinetics.steamEngine.SteamEngineBlock;
import net.minecraft.class_2350;
import net.minecraft.class_2350.class_2351;
import net.minecraft.class_243;
import net.minecraft.class_2680;
import net.minecraft.class_4587;

public class SteamEngineValueBox extends ValueBoxTransform.Sided {

    @Override
    protected boolean isSideActive(class_2680 state, class_2350 side) {
        class_2350 engineFacing = SteamEngineBlock.getFacing(state);
        if (engineFacing.method_10166() == side.method_10166())
            return false;

        float roll = 0;
        for (Pointing p : Pointing.values())
            if (p.getCombinedDirection(engineFacing) == side)
                roll = p.getXRotation();
        if (engineFacing == class_2350.field_11036)
            roll += 180;

        boolean recessed = roll % 180 == 0;
        if (engineFacing.method_10166() == class_2351.field_11052)
            recessed ^= state.method_11654(SteamEngineBlock.field_11177).method_10166() == class_2351.field_11048;

        return !recessed;
    }

    @Override
    public class_243 getLocalOffset(class_2680 state) {
        class_2350 side = getSide();
        class_2350 engineFacing = SteamEngineBlock.getFacing(state);

        float roll = 0;
        for (Pointing p : Pointing.values())
            if (p.getCombinedDirection(engineFacing) == side)
                roll = p.getXRotation();
        if (engineFacing == class_2350.field_11036)
            roll += 180;

        float horizontalAngle = AngleHelper.horizontalAngle(engineFacing);
        float verticalAngle = AngleHelper.verticalAngle(engineFacing);
        class_243 local = VecHelper.voxelSpace(8, 14.5, 9);

        local = VecHelper.rotateCentered(local, roll, class_2351.field_11051);
        local = VecHelper.rotateCentered(local, horizontalAngle, class_2351.field_11052);
        local = VecHelper.rotateCentered(local, verticalAngle, class_2351.field_11048);

        return local;
    }

    @Override
    public void rotate(class_2680 state, class_4587 ms) {
        class_2350 facing = SteamEngineBlock.getFacing(state);

        if (facing.method_10166() == class_2351.field_11052) {
            super.rotate(state, ms);
            return;
        }

        float roll = 0;
        for (Pointing p : Pointing.values())
            if (p.getCombinedDirection(facing) == getSide())
                roll = p.getXRotation();

        float yRot = AngleHelper.horizontalAngle(facing) + (facing == class_2350.field_11033 ? 180 : 0);
        TransformStack.of(ms).rotateYDegrees(yRot).rotateXDegrees(facing == class_2350.field_11033 ? -90 : 90).rotateYDegrees(roll);
    }

    @Override
    protected class_243 getSouthLocation() {
        return class_243.field_1353;
    }

}
